@@ -33,14 +33,24 @@ import java.util.logging.Level;
 import java.util.logging.Logger;
 import java.util.zip.GZIPInputStream;
 import java.util.zip.GZIPOutputStream;
+import javax.faces.FacesWrapper;
+import javax.faces.application.StateManager;
 
 import javax.faces.context.ExternalContext;
 import javax.faces.context.FacesContext;
+import javax.faces.lifecycle.ClientWindow;
+
 import org.apache.myfaces.application.StateCache;
+import org.apache.myfaces.application.viewstate.token.ServiceSideStateTokenProcessor;
+import org.apache.myfaces.application.viewstate.token.StateTokenProcessor;
 import org.apache.myfaces.buildtools.maven2.plugin.builder.annotation.JSFWebConfigParam;
+import org.apache.myfaces.shared.config.MyfacesConfig;
 import org.apache.myfaces.shared.renderkit.RendererUtils;
 import org.apache.myfaces.shared.util.MyFacesObjectInputStream;
 import org.apache.myfaces.shared.util.WebConfigParamUtils;
+import org.apache.myfaces.spi.ViewScopeProvider;
+import org.apache.myfaces.spi.ViewScopeProviderFactory;
+import org.apache.myfaces.view.ViewScopeProxyMap;
 
 class ServerSideStateCacheImpl extends StateCache<Object, Object>
 {
@@ -51,41 +61,22 @@ class ServerSideStateCacheImpl extends StateCache<Object, Object>
     
     public static final String RESTORED_SERIALIZED_VIEW_REQUEST_ATTR = 
         ServerSideStateCacheImpl.class.getName() + ".RESTORED_SERIALIZED_VIEW";
+    
+    public static final String RESTORED_SERIALIZED_VIEW_ID_REQUEST_ATTR = 
+        ServerSideStateCacheImpl.class.getName() + ".RESTORED_SERIALIZED_VIEW_ID";
+    public static final String RESTORED_SERIALIZED_VIEW_KEY_REQUEST_ATTR = 
+        ServerSideStateCacheImpl.class.getName() + ".RESTORED_SERIALIZED_VIEW_KEY";
 
     public static final String RESTORED_VIEW_KEY_REQUEST_ATTR = 
         ServerSideStateCacheImpl.class.getName() + ".RESTORED_VIEW_KEY";
     
-    /**
-     * Defines the amount (default = 20) of the latest views are stored in session.
-     * 
-     * <p>Only applicable if state saving method is "server" (= default).
-     * </p>
-     * 
-     */
-    @JSFWebConfigParam(defaultValue="20",since="1.1", classType="java.lang.Integer", group="state", tags="performance")
-    public static final String NUMBER_OF_VIEWS_IN_SESSION_PARAM = "org.apache.myfaces.NUMBER_OF_VIEWS_IN_SESSION";
+    public static final String NUMBER_OF_VIEWS_IN_SESSION_PARAM = MyfacesConfig.INIT_PARAM_NUMBER_OF_VIEWS_IN_SESSION;
 
-    /**
-     * Indicates the amount of views (default is not active) that should be stored in session between sequential
-     * POST or POST-REDIRECT-GET if org.apache.myfaces.USE_FLASH_SCOPE_PURGE_VIEWS_IN_SESSION is true.
-     * 
-     * <p>Only applicable if state saving method is "server" (= default). For example, if this param has value = 2 and 
-     * in your custom webapp there is a form that is clicked 3 times, only 2 views
-     * will be stored and the third one (the one stored the first time) will be
-     * removed from session, even if the view can
-     * store more sessions org.apache.myfaces.NUMBER_OF_VIEWS_IN_SESSION.
-     * This feature becomes useful for multi-window applications.
-     * where without this feature a window can swallow all view slots so
-     * the other ones will throw ViewExpiredException.</p>
-     */
-    @JSFWebConfigParam(since="2.0.6", classType="java.lang.Integer", group="state", tags="performance")
     public static final String NUMBER_OF_SEQUENTIAL_VIEWS_IN_SESSION_PARAM
-            = "org.apache.myfaces.NUMBER_OF_SEQUENTIAL_VIEWS_IN_SESSION";
+            = MyfacesConfig.INIT_PARAM_NUMBER_OF_SEQUENTIAL_VIEWS_IN_SESSION;
     
-    /**
-     * Default value for <code>org.apache.myfaces.NUMBER_OF_VIEWS_IN_SESSION</code> context parameter.
-     */
-    public static final int DEFAULT_NUMBER_OF_VIEWS_IN_SESSION = 20;
+    public static final int DEFAULT_NUMBER_OF_VIEWS_IN_SESSION = 
+            MyfacesConfig.INIT_PARAM_NUMBER_OF_VIEWS_IN_SESSION_DEFAULT;
 
     /**
      * Indicate if the state should be serialized before save it on the session. 
@@ -94,8 +85,11 @@ class ServerSideStateCacheImpl extends StateCache<Object, Object>
      * If <code>true</code> (default) the state will be serialized to a byte stream before it is written to the session.
      * If <code>false</code> the state will not be serialized to a byte stream.
      * </p>
+     * @deprecated 
      */
-    @JSFWebConfigParam(defaultValue="true",since="1.1", expectedValues="true,false", group="state", tags="performance")
+    @Deprecated
+    @JSFWebConfigParam(defaultValue="false",since="1.1", expectedValues="true,false", 
+        group="state", tags="performance", deprecated=true)
     public static final String SERIALIZE_STATE_IN_SESSION_PARAM = "org.apache.myfaces.SERIALIZE_STATE_IN_SESSION";
 
     /**
@@ -115,52 +109,11 @@ class ServerSideStateCacheImpl extends StateCache<Object, Object>
     public static final boolean DEFAULT_COMPRESS_SERVER_STATE_PARAM = true;
 
     /**
-     * Default value for <code>org.apache.myfaces.SERIALIZE_STATE_IN_SESSION</code> context parameter.
+     * Default value for <code>javax.faces.SERIALIZE_SERVER_STATE and 
+     * org.apache.myfaces.SERIALIZE_STATE_IN_SESSION</code> context parameter.
      */
-    public static final boolean DEFAULT_SERIALIZE_STATE_IN_SESSION = true;
-
-    /**
-     * Define the way of handle old view references(views removed from session), making possible to
-     * store it in a cache, so the state manager first try to get the view from the session. If is it
-     * not found and soft or weak ReferenceMap is used, it try to get from it.
-     * <p>
-     * Only applicable if state saving method is "server" (= default).
-     * </p>
-     * <p>
-     * The gc is responsible for remove the views, according to the rules used for soft, weak or phantom
-     * references. If a key in soft and weak mode is garbage collected, its values are purged.
-     * </p>
-     * <p>
-     * By default no cache is used, so views removed from session became phantom references.
-     * </p>
-     * <ul> 
-     * <li> off, no: default, no cache is used</li> 
-     * <li> hard-soft: use an ReferenceMap(AbstractReferenceMap.HARD, AbstractReferenceMap.SOFT)</li>
-     * <li> soft: use an ReferenceMap(AbstractReferenceMap.SOFT, AbstractReferenceMap.SOFT, true) </li>
-     * <li> soft-weak: use an ReferenceMap(AbstractReferenceMap.SOFT, AbstractReferenceMap.WEAK, true) </li>
-     * <li> weak: use an ReferenceMap(AbstractReferenceMap.WEAK, AbstractReferenceMap.WEAK, true) </li>
-     * </ul>
-     * 
-     */
-    @JSFWebConfigParam(defaultValue="off", expectedValues="off, no, hard-soft, soft, soft-weak, weak",
-                       since="1.2.5", group="state", tags="performance")
-    public static final String CACHE_OLD_VIEWS_IN_SESSION_MODE = "org.apache.myfaces.CACHE_OLD_VIEWS_IN_SESSION_MODE";
+    public static final boolean DEFAULT_SERIALIZE_STATE_IN_SESSION = false;
     
-    /**
-     * This option uses an hard-soft ReferenceMap, but it could cause a 
-     * memory leak, because the keys are not removed by any method
-     * (MYFACES-1660). So use with caution.
-     */
-    public static final String CACHE_OLD_VIEWS_IN_SESSION_MODE_HARD_SOFT = "hard-soft";
-    
-    public static final String CACHE_OLD_VIEWS_IN_SESSION_MODE_SOFT = "soft";
-    
-    public static final String CACHE_OLD_VIEWS_IN_SESSION_MODE_SOFT_WEAK = "soft-weak";
-    
-    public static final String CACHE_OLD_VIEWS_IN_SESSION_MODE_WEAK = "weak";
-    
-    public static final String CACHE_OLD_VIEWS_IN_SESSION_MODE_OFF = "off";
-
     /**
      * Allow use flash scope to keep track of the views used in session and the previous ones,
      * so server side state saving can delete old views even if POST-REDIRECT-GET pattern is used.
@@ -173,19 +126,18 @@ class ServerSideStateCacheImpl extends StateCache<Object, Object>
     public static final String USE_FLASH_SCOPE_PURGE_VIEWS_IN_SESSION
             = "org.apache.myfaces.USE_FLASH_SCOPE_PURGE_VIEWS_IN_SESSION";
 
-    public static final String RANDOM_KEY_IN_VIEW_STATE_SESSION_TOKEN_NONE = "none";
     public static final String RANDOM_KEY_IN_VIEW_STATE_SESSION_TOKEN_SECURE_RANDOM = "secureRandom";
     public static final String RANDOM_KEY_IN_VIEW_STATE_SESSION_TOKEN_RANDOM = "random";
     
     /**
      * Adds a random key to the generated view state session token.
      */
-    @JSFWebConfigParam(since="2.1.9, 2.0.15", expectedValues="secureRandom, random, none", 
-            defaultValue="none", group="state")
+    @JSFWebConfigParam(since="2.1.9, 2.0.15", expectedValues="secureRandom, random", 
+            defaultValue="secureRandom", group="state")
     public static final String RANDOM_KEY_IN_VIEW_STATE_SESSION_TOKEN_PARAM
             = "org.apache.myfaces.RANDOM_KEY_IN_VIEW_STATE_SESSION_TOKEN";
     public static final String RANDOM_KEY_IN_VIEW_STATE_SESSION_TOKEN_PARAM_DEFAULT = 
-            RANDOM_KEY_IN_VIEW_STATE_SESSION_TOKEN_NONE;
+            RANDOM_KEY_IN_VIEW_STATE_SESSION_TOKEN_SECURE_RANDOM;
 
     /**
      * Set the default length of the random key added to the view state session token.
@@ -229,6 +181,8 @@ class ServerSideStateCacheImpl extends StateCache<Object, Object>
     private boolean _numberOfSequentialViewsInSessionSet = false;
 
     private SessionViewStorageFactory sessionViewStorageFactory;
+    private CsrfSessionTokenFactory csrfSessionTokenFactory;
+    private StateTokenProcessor stateTokenProcessor;
 
     public ServerSideStateCacheImpl()
     {
@@ -248,8 +202,29 @@ class ServerSideStateCacheImpl extends StateCache<Object, Object>
         }
         else
         {
-            sessionViewStorageFactory = new CounterSessionViewStorageFactory(new CounterKeyFactory());
+            if (randomMode != null && !randomMode.isEmpty())
+            {
+                log.warning(RANDOM_KEY_IN_VIEW_STATE_SESSION_TOKEN_PARAM + " \""
+                        + randomMode + "\" is not supported (anymore)."
+                        + " Fallback to \"secureRandom\"");
+            }
+            sessionViewStorageFactory = new RandomSessionViewStorageFactory(
+                    new SecureRandomKeyFactory(facesContext));
         }
+        
+        String csrfRandomMode = WebConfigParamUtils.getStringInitParameter(facesContext.getExternalContext(),
+                RANDOM_KEY_IN_CSRF_SESSION_TOKEN_PARAM, 
+                RANDOM_KEY_IN_CSRF_SESSION_TOKEN_PARAM_DEFAULT);
+        if (RANDOM_KEY_IN_CSRF_SESSION_TOKEN_RANDOM.equals(csrfRandomMode))
+        {
+            csrfSessionTokenFactory = new RandomCsrfSessionTokenFactory(facesContext);
+        }
+        else
+        {
+            csrfSessionTokenFactory = new SecureRandomCsrfSessionTokenFactory(facesContext);
+        }
+        
+        stateTokenProcessor = new ServiceSideStateTokenProcessor();
     }
     
     //------------------------------------- METHODS COPIED FROM JspStateManagerImpl--------------------------------
@@ -285,7 +260,16 @@ class ServerSideStateCacheImpl extends StateCache<Object, Object>
             
             if (key == null )
             {
-                if (isUseFlashScopePurgeViewsInSession(context.getExternalContext()) && 
+                // Check if clientWindow is enabled and if the last view key is stored
+                // into session, so we can use it to chain the precedence in GET-GET
+                // cases.
+                ClientWindow clientWindow = context.getExternalContext().getClientWindow();
+                if (clientWindow != null)
+                {
+                    key = (SerializedViewKey) viewCollection.
+                            getLastWindowKey(context, clientWindow.getId());
+                }
+                else if (isUseFlashScopePurgeViewsInSession(context.getExternalContext()) && 
                     Boolean.TRUE.equals(context.getExternalContext().getRequestMap()
                             .get("oam.Flash.REDIRECT.PREVIOUSREQUEST")))
                 {
@@ -297,8 +281,45 @@ class ServerSideStateCacheImpl extends StateCache<Object, Object>
         
         SerializedViewKey nextKey = getSessionViewStorageFactory().createSerializedViewKey(
                 context, context.getViewRoot().getViewId(), getNextViewSequence(context));
-        viewCollection.add(context, serializeView(context, serializedView), nextKey, key);
+        // Get viewScopeMapId
+        ViewScopeProxyMap viewScopeProxyMap = null;
+        Object viewMap = context.getViewRoot().getViewMap(false);
+        if (viewMap != null)
+        {
+            while (viewMap != null)
+            {
+                if (viewMap instanceof ViewScopeProxyMap)
+                {
+                    viewScopeProxyMap = (ViewScopeProxyMap)viewMap;
+                    break;
+                }
+                else if (viewMap instanceof FacesWrapper)
+                {
+                    viewMap = ((FacesWrapper)viewMap).getWrapped();
+                }
+            }
 
+        }
+        if (viewScopeProxyMap != null)
+        {
+            ViewScopeProviderFactory factory = ViewScopeProviderFactory.getViewScopeHandlerFactory(
+                context.getExternalContext());
+            ViewScopeProvider handler = factory.getViewScopeHandler(context.getExternalContext());
+            viewCollection.put(context, serializeView(context, serializedView), nextKey, key,
+                    handler, viewScopeProxyMap.getViewScopeId());
+        }
+        else
+        {
+            viewCollection.put(context, serializeView(context, serializedView), nextKey, key);
+        }
+
+        ClientWindow clientWindow = context.getExternalContext().getClientWindow();
+        if (clientWindow != null)
+        {
+            //Update the last key generated for the current windowId in session map
+            viewCollection.putLastWindowKey(context, clientWindow.getId(), nextKey);
+        }
+        
         // replace the value to notify the container about the change
         sessionMap.put(SERIALIZED_VIEW_SESSION_ATTR, viewCollection);
     }
@@ -345,23 +366,56 @@ class ServerSideStateCacheImpl extends StateCache<Object, Object>
                 }
             }
 
-            nextViewSequence(context);
+            if (context.getPartialViewContext().isAjaxRequest() ||
+                context.getPartialViewContext().isPartialRequest())
+            {
+                // Save the information used to restore. The idea is use this information later
+                // to decide if it is necessary to generate a new view sequence or use the existing
+                // one.
+                attributeMap.put(RESTORED_SERIALIZED_VIEW_KEY_REQUEST_ATTR, sequence);
+                attributeMap.put(RESTORED_SERIALIZED_VIEW_ID_REQUEST_ATTR, viewId);
+            }
+            else
+            {
+                // Ensure a new sequence is used for the next view
+                nextViewSequence(context);
+            }
         }
         return serializedView;
     }
 
-    public Object getNextViewSequence(FacesContext context)
+    protected Object getNextViewSequence(FacesContext context)
     {
         Object sequence = context.getAttributes().get(RendererUtils.SEQUENCE_PARAM);
         if (sequence == null)
         {
-            sequence = nextViewSequence(context);
+            if (context.getPartialViewContext().isAjaxRequest() ||
+                context.getPartialViewContext().isPartialRequest())
+            {
+                String restoredViewId = (String) context.getAttributes().get(RESTORED_SERIALIZED_VIEW_ID_REQUEST_ATTR);
+                Object restoredKey = context.getAttributes().get(RESTORED_SERIALIZED_VIEW_KEY_REQUEST_ATTR);
+                if (restoredViewId != null && restoredKey != null)
+                {
+                    if (restoredViewId.equals(context.getViewRoot().getViewId()))
+                    {
+                        // The same viewId that was restored is the same that is being processed 
+                        // and the request is partial or ajax. In this case we can reuse the restored
+                        // key.
+                        sequence = restoredKey;
+                    }
+                }
+            }
+            
+            if (sequence == null)
+            {
+                sequence = nextViewSequence(context);
+            }
             context.getAttributes().put(RendererUtils.SEQUENCE_PARAM, sequence);
         }
         return sequence;
     }
 
-    public Object nextViewSequence(FacesContext facescontext)
+    protected Object nextViewSequence(FacesContext facescontext)
     {
         Object sequence = getKeyFactory(facescontext).generateKey(facescontext);
         facescontext.getAttributes().put(RendererUtils.SEQUENCE_PARAM, sequence);
@@ -444,9 +498,19 @@ class ServerSideStateCacheImpl extends StateCache<Object, Object>
     protected boolean isSerializeStateInSession(FacesContext context)
     {
         String value = context.getExternalContext().getInitParameter(
-                SERIALIZE_STATE_IN_SESSION_PARAM);
+                StateManager.SERIALIZE_SERVER_STATE_PARAM_NAME);
+        
         boolean serialize = DEFAULT_SERIALIZE_STATE_IN_SESSION;
         if (value != null)
+        {
+            serialize = value.toLowerCase().equals("true");
+            return serialize;
+        }
+        
+        // Fallback old parameter.
+        value = context.getExternalContext().getInitParameter(
+                SERIALIZE_STATE_IN_SESSION_PARAM);
+        if (value != null)
         {
            serialize = Boolean.valueOf(value);
         }
@@ -527,17 +591,7 @@ class ServerSideStateCacheImpl extends StateCache<Object, Object>
                     }
                 }
             }
-            catch (PrivilegedActionException e) 
-            {
-                log.log(Level.SEVERE, "Exiting deserializeView - Could not deserialize state: " + e.getMessage(), e);
-                return null;
-            }
-            catch (IOException e)
-            {
-                log.log(Level.SEVERE, "Exiting deserializeView - Could not deserialize state: " + e.getMessage(), e);
-                return null;
-            }
-            catch (ClassNotFoundException e)
+            catch (PrivilegedActionException | IOException | ClassNotFoundException e) 
             {
                 log.log(Level.SEVERE, "Exiting deserializeView - Could not deserialize state: " + e.getMessage(), e);
                 return null;
@@ -600,6 +654,7 @@ class ServerSideStateCacheImpl extends StateCache<Object, Object>
                 : getSerializedViewFromServletSession(facesContext, viewId, serverStateId);
     }
 
+    @Override
     public Object encodeSerializedState(FacesContext facesContext, Object serializedView)
     {
         return getKeyFactory(facesContext).encode(getNextViewSequence(facesContext));
@@ -627,9 +682,8 @@ class ServerSideStateCacheImpl extends StateCache<Object, Object>
     {
         if (!_numberOfSequentialViewsInSessionSet)
         {
-            _numberOfSequentialViewsInSession = WebConfigParamUtils.getIntegerInitParameter(
-                    externalContext, 
-                    NUMBER_OF_SEQUENTIAL_VIEWS_IN_SESSION_PARAM);
+            _numberOfSequentialViewsInSession = MyfacesConfig.getCurrentInstance(externalContext)
+                    .getNumberOfSequentialViewsInSession();
             _numberOfSequentialViewsInSessionSet = true;
         }
         return _numberOfSequentialViewsInSession;
@@ -645,4 +699,16 @@ class ServerSideStateCacheImpl extends StateCache<Object, Object>
     {
         return sessionViewStorageFactory;
     }
+
+    @Override
+    public String createCryptographicallyStrongTokenFromSession(FacesContext context)
+    {
+        return csrfSessionTokenFactory.createCryptographicallyStrongTokenFromSession(context);
+    }
+    
+    @Override
+    public StateTokenProcessor getStateTokenProcessor(FacesContext context)
+    {
+        return stateTokenProcessor;
+    }
 }
