@@ -30,22 +30,17 @@ import javax.faces.view.facelets.FaceletContext;
 import javax.faces.view.facelets.TagAttribute;
 import javax.faces.view.facelets.TagAttributeException;
 
-import org.apache.myfaces.util.ExternalSpecifications;
 import org.apache.myfaces.view.facelets.AbstractFaceletContext;
 import org.apache.myfaces.view.facelets.el.CompositeComponentELUtils;
 import org.apache.myfaces.view.facelets.el.ContextAwareTagMethodExpression;
 import org.apache.myfaces.view.facelets.el.ContextAwareTagValueExpression;
-import org.apache.myfaces.view.facelets.el.ContextAwareTagValueExpressionUEL;
 import org.apache.myfaces.view.facelets.el.ELText;
 import org.apache.myfaces.view.facelets.el.LocationMethodExpression;
 import org.apache.myfaces.view.facelets.el.LocationValueExpression;
-import org.apache.myfaces.view.facelets.el.LocationValueExpressionUEL;
 import org.apache.myfaces.view.facelets.el.ResourceELUtils;
 import org.apache.myfaces.view.facelets.el.ResourceLocationValueExpression;
-import org.apache.myfaces.view.facelets.el.ResourceLocationValueExpressionUEL;
 import org.apache.myfaces.view.facelets.el.TagMethodExpression;
 import org.apache.myfaces.view.facelets.el.TagValueExpression;
-import org.apache.myfaces.view.facelets.el.TagValueExpressionUEL;
 import org.apache.myfaces.view.facelets.el.ValueExpressionMethodExpression;
 
 /**
@@ -127,8 +122,8 @@ public final class TagAttributeImpl extends TagAttribute
      * If literal, return {@link Boolean#getBoolean(java.lang.String) Boolean.getBoolean(java.lang.String)} passing our
      * value, otherwise call {@link #getObject(FaceletContext, Class) getObject(FaceletContext, Class)}.
      * 
-     * @see Boolean#getBoolean(java.lang.String)
-     * @see #getObject(FaceletContext, Class)
+     * See Boolean#getBoolean(java.lang.String)
+     * See #getObject(FaceletContext, Class)
      * @param ctx
      *            FaceletContext to use
      * @return boolean value
@@ -149,8 +144,8 @@ public final class TagAttributeImpl extends TagAttribute
      * If literal, call {@link Integer#parseInt(java.lang.String) Integer.parseInt(String)}, otherwise call
      * {@link #getObject(FaceletContext, Class) getObject(FaceletContext, Class)}.
      * 
-     * @see Integer#parseInt(java.lang.String)
-     * @see #getObject(FaceletContext, Class)
+     * See Integer#parseInt(java.lang.String)
+     * See #getObject(FaceletContext, Class)
      * @param ctx
      *            FaceletContext to use
      * @return int value
@@ -190,9 +185,9 @@ public final class TagAttributeImpl extends TagAttribute
     /**
      * Create a MethodExpression, using this attribute's value as the expression String.
      * 
-     * @see ExpressionFactory#createMethodExpression(javax.el.ELContext, java.lang.String, java.lang.Class,
+     * See ExpressionFactory#createMethodExpression(javax.el.ELContext, java.lang.String, java.lang.Class,
      *      java.lang.Class[])
-     * @see MethodExpression
+     * See MethodExpression
      * @param ctx
      *            FaceletContext to use
      * @param type
@@ -257,7 +252,7 @@ public final class TagAttributeImpl extends TagAttribute
                 // (see VDL.retargetMethodExpressions for details)
                 
                 // check for params in the the MethodExpression
-                if (ExternalSpecifications.isUnifiedELAvailable() && this.value.contains("("))
+                if (this.value.contains("("))
                 {
                     // if we don't throw this exception here, another ELException will be
                     // thrown later, because #{cc.attrs.method(param)} will not work as a
@@ -300,15 +295,17 @@ public final class TagAttributeImpl extends TagAttribute
                 if ((this.capabilities & EL_CC) != 0)
                 {
                     Location currentLocation = getLocation();
-                    Location ccLocation = (Location) actx.getFaceletCompositionContext().
-                            getCompositeComponentFromStack().getAttributes().get(
-                                    CompositeComponentELUtils.LOCATION_KEY);
-                    if (ccLocation != null && !ccLocation.getPath().equals(currentLocation.getPath()))
+                    UIComponent cc = actx.getFaceletCompositionContext().getCompositeComponentFromStack();
+                    if (cc != null)
                     {
-                        // #{cc} from a template called from inside a composite component, disable caching on 
-                        // this expression. The reason is we need to change the Location object used as
-                        // reference as the one in the stack, and that depends on the template hierarchy.
-                        currentLocation = ccLocation;
+                        Location ccLocation = (Location) cc.getAttributes().get(CompositeComponentELUtils.LOCATION_KEY);
+                        if (ccLocation != null && !ccLocation.getPath().equals(currentLocation.getPath()))
+                        {
+                            // #{cc} from a template called from inside a composite component, disable caching on 
+                            // this expression. The reason is we need to change the Location object used as
+                            // reference as the one in the stack, and that depends on the template hierarchy.
+                            currentLocation = ccLocation;
+                        }
                     }
                     methodExpression = new LocationMethodExpression(currentLocation, methodExpression, 
                             actx.getFaceletCompositionContext().getCompositeComponentLevel());
@@ -363,7 +360,7 @@ public final class TagAttributeImpl extends TagAttribute
     /**
      * Delegates to getObject with Object.class as a param
      * 
-     * @see #getObject(FaceletContext, Class)
+     * See #getObject(FaceletContext, Class)
      * @param ctx
      *            FaceletContext to use
      * @return Object representation of this attribute's value
@@ -396,7 +393,7 @@ public final class TagAttributeImpl extends TagAttribute
     /**
      * If literal, then return our value, otherwise delegate to getObject, passing String.class.
      * 
-     * @see #getObject(FaceletContext, Class)
+     * See #getObject(FaceletContext, Class)
      * @param ctx
      *            FaceletContext to use
      * @return String value of this attribute
@@ -417,9 +414,9 @@ public final class TagAttributeImpl extends TagAttribute
      * If literal, simply coerce our String literal value using an ExpressionFactory, otherwise create a ValueExpression
      * and evaluate it.
      * 
-     * @see ExpressionFactory#coerceToType(java.lang.Object, java.lang.Class)
-     * @see ExpressionFactory#createValueExpression(javax.el.ELContext, java.lang.String, java.lang.Class)
-     * @see ValueExpression
+     * See ExpressionFactory#coerceToType(java.lang.Object, java.lang.Class)
+     * See ExpressionFactory#createValueExpression(javax.el.ELContext, java.lang.String, java.lang.Class)
+     * See ValueExpression
      * @param ctx
      *            FaceletContext to use
      * @param type
@@ -463,8 +460,8 @@ public final class TagAttributeImpl extends TagAttribute
     /**
      * Create a ValueExpression, using this attribute's literal value and the passed expected type.
      * 
-     * @see ExpressionFactory#createValueExpression(javax.el.ELContext, java.lang.String, java.lang.Class)
-     * @see ValueExpression
+     * See ExpressionFactory#createValueExpression(javax.el.ELContext, java.lang.String, java.lang.Class)
+     * See ValueExpression
      * @param ctx
      *            FaceletContext to use
      * @param type
@@ -529,28 +526,14 @@ public final class TagAttributeImpl extends TagAttribute
         {
             ExpressionFactory f = ctx.getExpressionFactory();
             ValueExpression valueExpression = f.createValueExpression(ctx, this.value, type);
-            
-            if (ExternalSpecifications.isUnifiedELAvailable())
+
+            if (actx.getFaceletCompositionContext().isWrapTagExceptionsAsContextAware())
             {
-                if (actx.getFaceletCompositionContext().isWrapTagExceptionsAsContextAware())
-                {
-                    valueExpression = new ContextAwareTagValueExpressionUEL(this, valueExpression);
-                }
-                else
-                {
-                    valueExpression = new TagValueExpressionUEL(this, valueExpression);
-                }
+                valueExpression = new ContextAwareTagValueExpression(this, valueExpression);
             }
             else
             {
-                if (actx.getFaceletCompositionContext().isWrapTagExceptionsAsContextAware())
-                {
-                    valueExpression = new ContextAwareTagValueExpression(this, valueExpression);
-                }
-                else
-                {
-                    valueExpression = new TagValueExpression(this, valueExpression);
-                }
+                valueExpression = new TagValueExpression(this, valueExpression);
             }
 
             // if the ValueExpression contains a reference to the current composite
@@ -565,38 +548,26 @@ public final class TagAttributeImpl extends TagAttribute
                 // the location of the composite component from the stack directly, but only when the path
                 // is different.
                 Location currentLocation = getLocation();
-                Location ccLocation = (Location) actx.getFaceletCompositionContext().
-                        getCompositeComponentFromStack().getAttributes().get(
-                                CompositeComponentELUtils.LOCATION_KEY);
-                if (ccLocation != null && !ccLocation.getPath().equals(currentLocation.getPath()))
+                UIComponent cc = actx.getFaceletCompositionContext().getCompositeComponentFromStack();
+                if (cc != null)
                 {
-                    // #{cc} from a template called from inside a composite component, disable caching on 
-                    // this expression. The reason is we need to change the Location object used as
-                    // reference as the one in the stack, and that depends on the template hierarchy.
-                    //cacheable = false;
-                    currentLocation = ccLocation;
-                }
-                if (ExternalSpecifications.isUnifiedELAvailable())
-                {
-                    valueExpression = new LocationValueExpressionUEL(currentLocation, valueExpression, 
-                            actx.getFaceletCompositionContext().getCompositeComponentLevel());
-                }
-                else
-                {
-                    valueExpression = new LocationValueExpression(currentLocation, valueExpression, 
-                            actx.getFaceletCompositionContext().getCompositeComponentLevel());
+                    Location ccLocation = (Location) cc.getAttributes().get(CompositeComponentELUtils.LOCATION_KEY);
+                    if (ccLocation != null && !ccLocation.getPath().equals(currentLocation.getPath()))
+                    {
+                        // #{cc} from a template called from inside a composite component, disable caching on 
+                        // this expression. The reason is we need to change the Location object used as
+                        // reference as the one in the stack, and that depends on the template hierarchy.
+                        //cacheable = false;
+                        currentLocation = ccLocation;
+                    }
                 }
+
+                valueExpression = new LocationValueExpression(currentLocation, valueExpression, 
+                        actx.getFaceletCompositionContext().getCompositeComponentLevel());
             }
             else if ((this.capabilities & EL_RESOURCE) != 0)
             {
-                if (ExternalSpecifications.isUnifiedELAvailable())
-                {
-                    valueExpression = new ResourceLocationValueExpressionUEL(getLocation(), valueExpression);
-                }
-                else
-                {
-                    valueExpression = new ResourceLocationValueExpression(getLocation(), valueExpression);
-                }
+                valueExpression = new ResourceLocationValueExpression(getLocation(), valueExpression);
             }
             
             
@@ -629,7 +600,7 @@ public final class TagAttributeImpl extends TagAttribute
     /*
      * (non-Javadoc)
      * 
-     * @see java.lang.Object#toString()
+     * See java.lang.Object#toString()
      */
     public String toString()
     {
