@@ -24,9 +24,6 @@ import org.apache.myfaces.test.base.AbstractJsfTestCase;
 
 /**
  * <code>HTMLEncoderTest</code> tests <code>org.apache.myfaces.shared.renderkit.html.util.HTMLEncoder</code>.
- * 
- * @author Michael Kurz (latest modification by $Author: lu4242 $)
- * @version $Revision: 1230285 $ $Date: 2012-01-11 22:23:28 +0000 (Wed, 11 Jan 2012) $
  */
 public class HTMLEncoderWriterTest extends AbstractJsfTestCase {
   private String stringNoSpecialChars = "Hello, this is MyFaces speaking!";
@@ -255,7 +252,7 @@ public class HTMLEncoderWriterTest extends AbstractJsfTestCase {
   {
       String cad1 = "http://myfaces.apache.org/hello.jsf?key1=val&key2=val2#id";
       String cad2 = "http://myfaces.apache.org/hello.jsf?key1=val&amp;key2=val2#id";
-      HTMLEncoder.encodeURIAtributte(sw, cad1,"UTF-8");
+      HTMLEncoder.encodeURIAttribute(sw, cad1,"UTF-8");
       String cad3 = sw.toString();
       assertEquals(cad2, cad3);      
   }
@@ -271,14 +268,14 @@ public class HTMLEncoderWriterTest extends AbstractJsfTestCase {
       // - From %7F ad infinitum
       String cad1 = "?key=\"%<>\\`{|}^\n "; //Omit %
       String cad2 = "?key=%22%25%3C%3E%5C%60%7B%7C%7D%5E%0A%20";
-      HTMLEncoder.encodeURIAtributte(sw, cad1,"UTF-8");
+      HTMLEncoder.encodeURIAttribute(sw, cad1,"UTF-8");
       String cad3 = sw.toString();
       assertEquals(cad2, cad3);
       
       String cad4 = "\"%<>\\`{|}^\n ";
       String cad5 = "%22%25%3C%3E%5C%60%7B%7C%7D%5E%0A%20";
       sw = new StringWriter();
-      HTMLEncoder.encodeURIAtributte(sw, cad4,"UTF-8");
+      HTMLEncoder.encodeURIAttribute(sw, cad4,"UTF-8");
       String cad6 = sw.toString();
       assertEquals(cad5, cad6);
       
@@ -292,7 +289,7 @@ public class HTMLEncoderWriterTest extends AbstractJsfTestCase {
 	  byte [] array = new byte[]{(byte)0xFC};
       String cad1 = new String(array,"ISO-8859-1");//+(char)0xC3BC;//"http://myfaces.apache.org/heüll o.jsf?key=val#id";
       String cad2 = "%C3%BC";//"http://myfaces.apache.org/he%FCll%20o.jsf?key=val#id";
-      HTMLEncoder.encodeURIAtributte(sw, cad1,"UTF-8");
+      HTMLEncoder.encodeURIAttribute(sw, cad1,"UTF-8");
       String cad3 = sw.toString();
       assertEquals(cad2, cad3);
 
@@ -309,13 +306,13 @@ public class HTMLEncoderWriterTest extends AbstractJsfTestCase {
       //               %21   %24   %26   %27   %28   %29   %2A   %2B   %2C   %3B   %3D
       
       String cad1 = "?key=:/[]@!$'()*+,;="; //Omit &
-      HTMLEncoder.encodeURIAtributte(sw, cad1,"UTF-8");
+      HTMLEncoder.encodeURIAttribute(sw, cad1,"UTF-8");
       String cad2 = sw.toString();
       assertEquals(cad1, cad2);
       
       String cad7 = ":/[]@!$&'()*+,;=";
       sw = new StringWriter(40);
-      HTMLEncoder.encodeURIAtributte(sw, cad7,"UTF-8");
+      HTMLEncoder.encodeURIAttribute(sw, cad7,"UTF-8");
       String cad8 = sw.toString();
       assertEquals(cad7, cad8);
   }
@@ -327,19 +324,19 @@ public class HTMLEncoderWriterTest extends AbstractJsfTestCase {
       // underscore (%5F), or tilde (%7E) should not be created by URI
       // producers...."
       String cad1 = "?key=ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-._~";
-      HTMLEncoder.encodeURIAtributte(sw, cad1,"UTF-8");
+      HTMLEncoder.encodeURIAttribute(sw, cad1,"UTF-8");
       String cad2 = sw.toString();
       assertEquals(cad1, cad2);
       
       String cad3 = "#somefile?key=ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-._~";
       sw = new StringWriter(40);
-      HTMLEncoder.encodeURIAtributte(sw, cad3,"UTF-8");
+      HTMLEncoder.encodeURIAttribute(sw, cad3,"UTF-8");
       String cad4 = sw.toString();
       assertEquals(cad3, cad4);
       
       String cad5 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-._~";
       sw = new StringWriter(40);
-      HTMLEncoder.encodeURIAtributte(sw, cad5,"UTF-8");
+      HTMLEncoder.encodeURIAttribute(sw, cad5,"UTF-8");
       String cad6 = sw.toString();
       assertEquals(cad5, cad6);
   }
@@ -358,7 +355,7 @@ public class HTMLEncoderWriterTest extends AbstractJsfTestCase {
       String cad11 = new String(array11,"UTF-8") + ((char)(0xFF))+((char)(0x100));
       String cad12 = "%C2%A1%C2%A2%C2%A3%C2%A4%C2%A5%C2%A6%C2%A7%C2%A8%C2%A9%C2%AA%C2%AB%C2%AC%C2%AD"+
                      "%C2%AE%C2%AF%C2%B0%C2%B1%C3%BF%C4%80";
-      HTMLEncoder.encodeURIAtributte(sw, cad11,"UTF-8");
+      HTMLEncoder.encodeURIAttribute(sw, cad11,"UTF-8");
       String cad13 = sw.toString();
       assertEquals(cad12, cad13);
       
@@ -366,12 +363,12 @@ public class HTMLEncoderWriterTest extends AbstractJsfTestCase {
       String cad2 = "?key=%C2%A1%C2%A2%C2%A3%C2%A4%C2%A5%C2%A6%C2%A7%C2%A8%C2%A9%C2%AA%C2%AB%C2%AC%C2%AD"+
                      "%C2%AE%C2%AF%C2%B0%C2%B1%C3%BF%C4%80";
       sw = new StringWriter(40);
-      HTMLEncoder.encodeURIAtributte(sw, cad1,"UTF-8");
+      HTMLEncoder.encodeURIAttribute(sw, cad1,"UTF-8");
       String cad3 = sw.toString();
       assertEquals(cad2, cad3);
             
       //String cad14 = "http://myfaces.apache.org/page.jsf?key="+((char)0xFF)+((char)0x100);
-      //String cad15 = HTMLEncoder.encodeURIAtributte(cad14,false);
+      //String cad15 = HTMLEncoder.encodeURIAttribute(cad14,false);
       //assertEquals(cad14,cad15);
   }
     
